# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
from os import path
import sys
from importlib import import_module
cleandocdir, _none = path.split(path.abspath(__file__))
sys.path.insert(0, path.split(cleandocdir)[0])
cd = import_module("cleandoc")
"""

from argparse import ArgumentParser
import cleandoc as cd


def cli_checks(pypath: str, dirpath: str, noclean: bool, nodoc: bool):
    """Check that command line args are compatible

    Parameters
    ----------
    pypath : str
    dirpath : str
    noclean : bool
    nodoc : bool
    """
    if len(pypath) > 0 and len(dirpath) > 0:
        raise SyntaxError("File and Directory were both specified. Only specify one.")
    if len(pypath) == 0 and len(dirpath) == 0:
        raise SyntaxError(
            "Neither File or Directory were specified. Please specify one."
        )
    if len(pypath) > 0 and noclean:
        raise SyntaxError("File was specified with -noclean so nothing occured.")
    if noclean and nodoc:
        raise SyntaxError(
            "-noclean and -nodoc options were specified so nothing occured"
        )


def cli_file(pypath: str, write: bool):
    """Run clean_pyfile

    Parameters
    ----------
    pypath : str
        _description_
    write : bool
        _description_
    ignore : bool
        _description_
    """
    if len(pypath) > 0:
        cd.clean_pyfile(pypath, write=write)


def cli_dir(dirpath: str, write: bool, ignore: bool, noclean: bool, nodoc: bool):
    """Run clean_all, gen_docs, or cleandoc_all depending on args.

    Parameters
    ----------
    dirpath : str
    ignore : bool
    write : bool
    noclean : bool
    nodoc : bool
    """
    if len(dirpath) > 0 and nodoc:
        cd.clean_all(dirpath, write=write, ignore=ignore)
    if len(dirpath) > 0 and noclean:
        cd.gen_docs(dirpath)
    if len(dirpath) > 0:
        cd.cleandoc_all(dirpath, write=write, ignore=ignore)


def main():
    """Run full command line process"""
    desc = "Run automated cleaning and/or documentation of python code"
    fileh = "Python (.py) file to clean"
    dirh = "Directory containing Python (.py) files to clean and/or document"
    writeh = "Flag to write changes to files in-place"
    ignoreh = "Flag to continue through warnings"
    nch = "Flag to prevent cleaning of py files"
    ndh = "Flag to prevent html doc creation"

    parser = ArgumentParser(prog="cleandoc", description=desc)
    parser.add_argument("-file", "-f", default="", help=fileh)
    parser.add_argument("-dir", "-d", default="", help=dirh)
    parser.add_argument("-write", "-w", action="store_true", help=writeh)
    parser.add_argument("-ignore", "-i", action="store_true", help=ignoreh)
    parser.add_argument("-noclean", "-nc", action="store_true", help=nch)
    parser.add_argument("-nodoc", "-nd", action="store_true", help=ndh)

    args = vars(parser.parse_args())
    print(f"\nCommand Line Args:\n{args}\n")
    pypath, dirpath, write, ignore, noclean, nodoc = [args[key] for key in args.keys()]

    cli_checks(pypath, dirpath, noclean, nodoc)
    cli_file(pypath, write)
    cli_dir(dirpath, write, ignore, noclean, nodoc)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

import logging
from .helper import run_capture_out, format_header, check_for_pkg

# check for packages called in subprocess commands in this module
for pkg in ["black", "pylint", "mypy"]:
    check_for_pkg(pkg)


def run_black(pyfilepath: str, write: bool = True):
    """Run "black" command on .py file via subprocess and check output.
    Black is a python package which auto-formats python code according to PEP8.

    Parameters
    ----------
    pyfilepath : str
        Full path to python (.py) file
    write : bool
        True = write the auto-formatted code in-place
    Returns
    -------
    str
        Summary of command outputs
    """
    # Auto-format code with black
    black_out, black_err = run_capture_out(["black", pyfilepath, "--diff"])
    if write:
        black_out, black_err = run_capture_out(["black", pyfilepath])
    black_str = f"{format_header('Black Output')}\n{black_out}\n{black_err}\n"
    if "1 file" in black_str and "left unchanged" in black_str:
        return ""
    logger = logging.getLogger("cleandoc")
    logger.info(black_str)
    return black_str


def run_pylint(pyfilepath: str):
    """Run "pylint" command on .py file via subprocess and check output.
    Pylint is a python package which checks for code cleanliness.

    Parameters
    ----------
    pyfilepath : str
        Full path to python (.py) file
    Returns
    -------
    str
        Summary of command outputs
    """
    # Check code cleanliness with pylint
    pylint_out, pylint_err = run_capture_out(["pylint", pyfilepath])
    pylint_str = f"{format_header('Pylint Output')}\n{pylint_out}\n{pylint_err}"
    if "Your code has been rated at 10.00/10" in pylint_str:
        return ""
    logger = logging.getLogger("cleandoc")
    logger.info(pylint_str)
    return pylint_str


def run_mypy(pyfilepath: str):
    """Run "mypy" command on .py file via subprocess and check output.
    Mypy is a python package which staticly checks variable types
    according to type hints.

    Parameters
    ----------
    pyfilepath : str
        Full path to python (.py) file
    Returns
    -------
    str
        Summary of command outputs
    """
    # Check variable type hints with mypy
    mypy_args = ["mypy", pyfilepath, "--check-untyped-defs", "--ignore-missing-imports"]
    mypy_out, mypy_err = run_capture_out(mypy_args)
    mypy_str = f"{format_header('Mypy Output')}\n{mypy_out}\n{mypy_err}"
    if "Success: no issues found" in mypy_str:
        return ""
    logger = logging.getLogger("cleandoc")
    logger.info(mypy_str)
    return mypy_str


# if __name__ == "__main__":



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from subprocess import run
from re import findall
import logging
from .helper import run_capture_out, format_header, check_for_pkg, findall_infile

# check for packages called in subprocess commands in this module
check_for_pkg("doq")


def run_doq(pyfilepath: str, formatter: str = "numpy", write: bool = True):
    """Auto-Generate Docstrings with doq (py-doq) package. Doq command is
    executed via subprocess command to create docstrings where missing.

    Parameters
    ----------
    pyfilepath : str
        Full path to python (.py) file
    formatter : str
        ["numpy","google","sphinx"] : See doq documentation for formatting options
    write : bool
        True = write the auto-gen docstrings in-place

    Returns
    -------
    str
        Summary of command outputs
    """
    # Generate Simple Docstrings with doq where they dont exists
    doq_out, doq_err = run_capture_out(
        ["doq", "-f", pyfilepath, f"--formatter={formatter}"]
    )
    if (len(doq_out) + len(doq_err)) == 0:
        return ""
    if write:
        run(["doq", "-f", pyfilepath, "-w", f"--formatter={formatter}"], check=True)
        doq_str = f"{format_header('Doq Output')}\n\n\
            Simple Docstrings Added. Please Complete Them!\n\n"
    else:
        results = findall(r'("""(.|\n|\r)*?""")', doq_out + doq_err)
        doq_strings = "\n".join([result[0] for result in results])
        doq_str = f"{format_header('Doq Output')}\n{doq_strings}\n"
    logger = logging.getLogger("cleandoc")
    logger.info(doq_str)
    return doq_str


def check_docstrings(pyfilepath: str):
    """Check if Docstrings were not edited after being auto-generated.

    Parameters
    ----------
    pyfilepath : str
        Full path of python (.py) file

    Returns
    -------
    str
        Summary of command outputs
    """
    desc_regex = r'def ([^(]*).*\r*\n *("""(.*?)(.|\n|\r)*?""")'
    var_regex = r" +([a-zA-Z0-9_]+) :.*\r*\n +([a-zA-Z0-9_]+)"
    desc_search = findall_infile(desc_regex, pyfilepath)
    check_str = ""
    for desc_found in desc_search:
        funname, docstring, desc = desc_found[0:3]
        check_sum = funname == (desc + ".")
        var_search = findall(var_regex, docstring)
        for var_found in var_search:
            check_sum += var_found[0] == var_found[1]
        if check_sum > 0:
            check_str += docstring + "\n"
    if len(check_str) == 0:
        return ""
    logger = logging.getLogger("cleandoc")
    check_str = (
        f"{format_header('Check Docstring Output')}\n\n\
        Complete the following auto-generated docstrings!\n\n"
        + check_str
    )
    logger.info(check_str)
    return check_str


# if __name__ == "__main__":



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from os import path, mkdir
from shutil import rmtree, copytree, copyfile
import logging
from importlib import reload
import webbrowser
from typing import Union, List
from .clean import run_black, run_pylint, run_mypy
from .doq import run_doq, check_docstrings
from . import helper as ch
from .sphinx import run_sphinx_all, get_release

reload(logging)


def cleandoc_all(
    searchpath: str, ignore: bool = False, write: bool = True, openhtml: bool = True
):
    """Run clean_all and gen_docs functions. Check modified files since last
    document generation to skip checking of some files. Open docs in browser
    after creation or checking.

    Parameters
    ----------
    searchpath : str
        directory of python package (nested dirs of modules)
    ignore : bool
        keyword argument passed to clean_all function
    """
    searchpath = path.abspath(searchpath)
    skiplist = ch.get_clean_pyfiles("cleandoc_log.txt")
    createdocs = ch.check_modified_since_docs(searchpath, "cleandoc_log.txt")
    ch.config_log("cleandoc_log.txt")
    clean_all(searchpath, ignore=ignore, write=write, skip=skiplist)
    mainpage = gen_docs(searchpath, create=createdocs)
    if openhtml is True:
        webbrowser.open(mainpage)
    logging.shutdown()


def clean_all(
    searchpath: str,
    ignore: bool = False,
    write: bool = True,
    skip: Union[bool, List[str]] = False,
):
    """Run clean_pyfile function on all .py files in searchpath.

    Parameters
    ----------
    searchpath : str
        Directory to search in all nested folders for .py files
    ignore : bool
        True to ignore Syntax warnings found, False to raise them
    write : bool
        keyword argument passed to clean_pyfile function
    skip : Union[bool, List[str]]
        List of .py files to skip cleaning.
        Or True to find list of clean pyfiles within function.
    """
    searchpath = path.abspath(searchpath)
    if not path.isdir(searchpath):
        raise FileNotFoundError("Searchpath does not exist: " + str(searchpath))
    if skip is True:
        skip = ch.get_clean_pyfiles("cleandoc_log.txt")
    elif skip is False:
        skip = []
    _none1, pyfilelist = ch.find_pyfiles(searchpath)
    logger = ch.config_log("cleandoc_log.txt")
    for i, pyfile in enumerate(pyfilelist):
        _none2, pyname = path.split(pyfile)
        if pyfile in skip:  # type: ignore
            header = f"Skipping File ({i+1}/{len(pyfilelist)}): {pyname}"
            headerstr = f"{ch.format_header(header, repeat_char='o')}\n"
            logger.info(headerstr)
            logger.info("File is Clean: %s\n", pyfile)
            continue
        header = f"Checking File ({i+1}/{len(pyfilelist)}): {pyname}"
        headerstr = f"{ch.format_header(header, repeat_char='o')}\n"
        logger.info(headerstr)
        summary = clean_pyfile(pyfile, write=write)
        if (not ignore) and (len(summary) > 0):
            logger.error("%s\n", pyfile)
            logging.shutdown()
            raise SyntaxWarning(f"{pyfile}\n{summary}")
        if len(summary) == 0:
            logger.info("File is Clean: %s\n", pyfile)


def clean_pyfile(pyfilepath: str, write: bool = True):
    """Clean a .py file by checking docstrings then running doq, black,
    pylint, and mypy.

    Parameters
    ----------
    pyfilepath : str
        Full path of python (.py) file
    Returns
    -------
    str
        Summary of all command outputs concatenated together
    """
    if not path.isfile(pyfilepath):
        raise FileNotFoundError("Pyfilepath does not exist: " + str(pyfilepath))
    ch.config_log("cleandoc_log.txt")
    pyfilepath = path.abspath(pyfilepath)
    realpath = path.realpath(pyfilepath)
    summary = check_docstrings(realpath)
    summary += run_doq(realpath, write=write)
    summary += run_black(realpath, write=write)
    summary += run_pylint(realpath)
    summary += run_mypy(realpath)
    return summary


def gen_docs(pkgpath: str, create: bool = True):
    """Auto-generate sphinx html documentation for a python package.

    Parameters
    ----------
    pkgpath : str
        Full path to directory containing all python modules to document.
        Directory name will be used as package name.
    create : bool
        True to create docs or False to skip and log their location
    Returns
    -------
    str
        Path of index.html file, the home page of the sphinx docs
    """
    pkgpath = path.abspath(pkgpath)
    if not path.isdir(pkgpath):
        raise FileNotFoundError("Pkgpath does not exist: " + str(pkgpath))
    logger = ch.config_log("cleandoc_log.txt")
    basepath, pkgname = path.split(pkgpath)
    docs = path.join(basepath, "docs")
    indexpath = path.join(docs, "index.html")
    if create is False:
        logger.info("%s\n", ch.format_header("Skipping Gen Docs", repeat_char="o"))
        logger.info("Docs Location: %s\n", docs)
        return indexpath
    logger.info("%s\n", ch.format_header("Gen Docs Output", repeat_char="o"))
    logger.debug("    pkgpath: %s", pkgpath)
    docpath = path.join(basepath, f"_{pkgname}_working_docs")
    confpath = path.join(docpath, "source", "conf.py")
    confpath_old = path.join(docs, "conf.txt")
    release = get_release(confpath_old)
    if path.exists(docpath):
        rmtree(docpath)
    mkdir(docpath)
    run_sphinx_all(docpath, confpath, pkgpath, release)
    htmlpath = path.join(docpath, "build", "html")
    if path.exists(docs):
        rmtree(docs)
    copytree(htmlpath, docs)
    copyfile(confpath, confpath_old)
    rmtree(docpath)
    logger.info("Docs Location: %s\n", docs)
    return indexpath


# if __name__ == "__main__":
#    scriptdir, scriptname = path.split(__file__)
#    cleandoc_all(scriptdir)



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from os import path, sep, remove
from getpass import getuser
from re import findall, sub
import logging
from .helper import check_for_pkg, find_pyfiles, run_capture_out, format_header

# check for packages called in subprocess commands in this module
for pkg in ["sphinx", "sphinx_rtd_theme"]:
    check_for_pkg(pkg)


def run_sphinx_all(docpath: str, confpath: str, pkgpath: str, release: str):
    """Run 4 functions sequentially in sphinx module including run_apidoc,
    edit_conf, edit_index, and run_make. Automakes creation of sphinx html documents for
    a python package (aka nested folder structure of python modules.)

    Parameters
    ----------
    docpath : str
        Full path to directory where sphinx docs will be created
    confpath : str
        Full path to sphinx conf.py file
    pkgpath : str
        Full path to directory containing all python modules to document.
        Directory name will be used as package name.
    release : str
        Release number (version) of package
    """

    basepath, pkgname = path.split(pkgpath)
    summary = run_quickstart(docpath, pkgname, release)
    srcpath = path.join(docpath, "source")
    pathlist, _none2 = find_pyfiles(pkgpath)
    logger = logging.getLogger("cleandoc")
    logger.debug("    pathlist: %s", pathlist)
    for pypath in pathlist:
        initpath = path.join(pypath, "__init__.py")
        if not path.exists(initpath):
            with open(initpath, "w", encoding="ascii") as initfile:
                initfile.write("")
    summary += run_apidoc(srcpath, pkgpath)
    edit_conf(confpath, [basepath, pkgpath] + pathlist)
    edit_index(srcpath, pkgname)
    summary += run_make(docpath)
    if len(summary) > 0:
        raise SyntaxError(summary)


def run_quickstart(docpath: str, pkgname: str, release: str) -> str:
    """Run "sphinx-quickstart" command via subprocess and check output.
    Creates files to begin sphinx documentation.

    Parameters
    ----------
    docpath : str
        Full path to directory where sphinx docs will be created
    pkgname : str
        Name of python package to document
    release : str
        Release number (version) of package
    Returns
    -------
    str
        Shell output from "sphinx-quickstart" command
    """
    qs_args = [
        "sphinx-quickstart",
        docpath,
        "--sep",
        "-p",
        pkgname,
        "-a",
        getuser(),
        "-r",
        release,
        "-v",
        release,
        "-l",
        "English",
    ]
    logger = logging.getLogger("cleandoc")
    logger.debug(" ".join(qs_args))
    qs_out, qs_err = run_capture_out(qs_args)
    qs_str = f"{format_header('Sphinx Quickstart Output')}\n{qs_out}\n{qs_err}\n"
    if ("error" in qs_str.lower()) or ("warning" in qs_str.lower()):
        logger.info(qs_str)
        return qs_str
    return ""


def run_apidoc(srcpath: str, pkgpath: str) -> str:
    """Run "sphinx-apidoc" command via subprocess and check output.
    Creates an .rst file for each .py file found in nested directories.
    RST files can be later used to create an html page for each module.

    Parameters
    ----------
    srcpath : str
        Full path of sphinx "source" directory (created from sphinx-quickstart)
    pkgpath : str
        Full path to directory containing all python modules to document.
    Returns
    -------
    str
        Shell output from "sphinx-apidoc" command
    """
    apidoc_args = ["sphinx-apidoc", "-M", "-o", srcpath, pkgpath]
    logger = logging.getLogger("cleandoc")
    logger.debug(" ".join(apidoc_args))
    apidoc_out, apidoc_err = run_capture_out(apidoc_args)
    apidoc_str = (
        f"{format_header('Sphinx Apidoc Output')}\n{apidoc_out}\n{apidoc_err}\n"
    )
    if ("error" in apidoc_str.lower()) or ("warning" in apidoc_str.lower()):
        logger.info(apidoc_str)
        return apidoc_str
    return ""


def run_make(docpath: str) -> str:
    """Run "make html" command via subprocess and check output.
    Creates html documents from sphinx .rst files.

    Parameters
    ----------
    docpath : str
        Full path to directory where sphinx docs will be created
    Returns
    -------
    str
        Shell output from "make html" command
    """
    make_args = ["cd", f"{docpath}", "&&", "make", "html"]
    logger = logging.getLogger("cleandoc")
    logger.debug(" ".join(make_args))
    make_out, make_err = run_capture_out(make_args, shell=True)
    make_str = f"{format_header('Sphinx Make Output')}\n{make_out}\n{make_err}\n"
    if ("error" in make_str.lower()) or ("warning" in make_str.lower()):
        logger.info(make_str)
        return make_str
    return ""


def get_release(confpath: str) -> str:
    """Get the previous release number from last sphinx conf.py file.
    If no conf.py file is found, release "0.0.1" is returned.
    If previuos release is found, "0.0.+1" is returned.

    Parameters
    ----------
    confpath : str
        Full path to sphinx conf.py file

    Returns
    -------
    release : str
        Release number of package for current sphinx documentation.
    """

    if not path.exists(confpath):
        return "0.0.1"
    with open(confpath, "r", encoding="ascii") as conffile:
        conftext = conffile.readlines()
    release_found = findall(r"release = '([\d\.]*)'", "".join(conftext))
    if len(release_found) == 0:
        return "0.0.1"
    release = release_found[0]
    logger = logging.getLogger("cleandoc")
    logger.debug("    last sphinx release found: %s", release)
    release_split = release.split(".")
    release_split[2] = str(int(release_split[2]) + 1)
    release = ".".join(release_split)
    return release


def edit_conf(confpath: str, pathlist: list[str]):
    """Edit sphinx conf.py file by running add_conf_paths and then
    add_conf_settings function.

    Parameters
    ----------
    confpath : str
        Full path to sphinx conf.py file
    pathlist : list[str]
        List of full paths to add to conf.py file so all modules can be found.
    """
    with open(confpath, "r", encoding="ascii") as conffile:
        conflines_orig = conffile.readlines()
    conflines = add_conf_paths(conflines_orig, pathlist)
    conftext_orig = "".join(conflines)
    conftext = add_conf_settings(conftext_orig)
    with open(confpath, "w", encoding="ascii") as conffile:
        conffile.write(conftext)


def add_conf_paths(conflines: list[str], pathlist: list[str]):
    """Add path.insert() lines into beginning of conf.py file
    so all modules can be found by sphinx.

    Parameters
    ----------
    conflines : list[str]
        List of text file lines in conf.py file (from readlines)
    pathlist : list[str]
        List of paths to append to the system path
    Returns
    -------
    str
        conflines after inserting path.insert lines
    """
    importlines = ["from sys import path\n"]
    fixpaths = [dirpath.replace(sep, "/") for dirpath in pathlist]
    pathlines = [f'path.insert(0, "{fixpath}")\n' for fixpath in fixpaths]
    conflines_out = importlines + pathlines + conflines
    return conflines_out


def add_conf_settings(conftext: str) -> str:
    """Add extensions and Napoleon settings to sphinx conf.py file.

    Parameters
    ----------
    conftext : str
        Full text contents of sphinx conf.py file.
    Returns
    -------
    str
        Edited text contents of conf.py file.

    """
    sphinxexts = """
extensions = ['sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
    'sphinx.ext.napoleon',
    'sphinx_rtd_theme',]

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = True
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True
"""
    conftext = sub(r"extensions = \[\]", sphinxexts, conftext)
    conftext = sub(
        r"html_theme = 'alabaster'", "html_theme = 'sphinx_rtd_theme'", conftext
    )
    return conftext


def edit_index(srcpath: str, pkgname: str):
    """Edit index.rst file which controls the main page of html docs.

    Parameters
    ----------
    srcpath : str
        Full path of sphinx "source" directory (created from sphinx-quickstart)
    """
    add_modules = f":caption: Contents:\n    \n   {pkgname}"
    indexpath = path.join(srcpath, "index.rst")
    modulespath = path.join(srcpath, "modules.rst")
    if path.exists(modulespath):
        remove(modulespath)
    with open(indexpath, "r", encoding="ascii") as indexfile:
        indexlines = indexfile.readlines()
    indextext = "".join(indexlines)
    indextext = sub(r":caption: Contents:", add_modules, indextext)
    indextext = sub(r":maxdepth: 4", ":maxdepth: 10", indextext)
    with open(indexpath, "w", encoding="ascii") as indexfile:
        indexfile.write(indextext)


# if __name__ == "__main__":



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from subprocess import run
from math import floor, ceil
from importlib.util import find_spec
from typing import Tuple
from os import path, walk, remove
from datetime import datetime
from re import findall
import logging
from sys import stdout


def check_for_pkg(packagename: str):
    """Use importlib find_spec function to check if package is installed.

    Parameters
    ----------
    packagename : str
        Name of python package to check for
    """
    spec = find_spec(packagename)
    if spec is None:
        errorstr = (
            f"{packagename} is not installed! Please "
            f"install the package before rerunning the script."
        )
        raise ModuleNotFoundError(errorstr)


def run_capture_out(cmd: list[str], shell: bool = False) -> Tuple[str, str]:
    """Run subprocess command and return the stdout and stderr.

    Parameters
    ----------
    cmd : list[str]
        Pass list of shell commands to subprocess.run
    shell : bool
        Pass shell keyword argument to subprocess.run

    Returns
    -------
    stdout  : str
        Standard Output returned from shell
    stderr : str
        Standard Error returned from shell

    """
    proc = run(
        cmd,
        capture_output=True,
        encoding="utf-8",
        check=False,
        shell=shell,
        errors="ignore",
    )
    return proc.stdout, proc.stderr


def format_header(name: str, repeat_char: str = "-", linelen: int = 68) -> str:
    """Format a string header for printing or logging.

    Parameters
    ----------
    name : str
        String to include in middle of header
    repeat_char : str
        Character to repeat before and after name
    linelen : int
        Total length of string to create

    Returns
    -------
    header : str
        Full line string with name between repeated characters

    """
    start = repeat_char * floor((linelen - len(name) - 2) / 2)
    end = repeat_char * ceil((linelen - len(name) - 2) / 2)
    header = f"{start} {name} {end}"
    return header


def find_pyfiles(searchpath: str) -> Tuple[list[str], list[str]]:
    """Find all .py files in nested directories.

    Parameters
    ----------
    searchpath : str
        Full path to search through for .py files

    Returns
    -------
    pathlist : list[str]
        List of full paths to all directories containing .py files directly
    filelist : list[str]
        List of full paths to all .py files found

    """
    pathlist = []
    filelist = []
    for root, _none1, files in walk(searchpath):
        contains_py = False
        for filename in files:
            _none2, fileext = path.splitext(filename)
            if ".py" == fileext:
                contains_py = True
                filepath = path.join(root, filename)
                filelist.append(filepath)
        if contains_py:
            pathlist.append(root)
    return pathlist, filelist


def check_modified(filepath: str, timestr: str) -> bool:
    """Check if a file was modified since a certain date and time.

    Parameters
    ----------
    filepath : str
        Full path of file to check
    timestr : str
        String of time to check against, formatted as "%d-%m-%y %H:%M:%S"

    Returns
    -------
    bool
        True if the file has been modified since the timestring.

    """
    if not path.exists(filepath):
        return False
    checktime = datetime.strptime(timestr, "%d-%m-%y %H:%M:%S")
    checkstamp = datetime.timestamp(checktime)
    editstamp = path.getmtime(filepath)
    return editstamp > checkstamp


def check_modified_since_docs(searchpath: str, logpath: str) -> bool:
    """Check if any python files in nested directory were modified since
    html documents were created.

    Parameters
    ----------
    logpath : str
        Full path of previous log file output by cleandoc
    htmlpath : str
        Full path to directory containing previously generated html docs

    Returns
    -------
    bool
        True if modifed since doc creation

    """
    _none, filelist = find_pyfiles(searchpath)
    regex_docs = r"(\d\d-\d\d-\d\d \d\d:\d\d:\d\d).*Docs Location: (.*)"
    results_docs = findall_infile(regex_docs, logpath, skip_exist=True)
    if len(results_docs) == 0:
        return True
    doctime = results_docs[0][0]
    for filepath in filelist:
        if check_modified(filepath, doctime):
            return True
    return False


def findall_infile(regex: str, filepath: str, skip_exist: bool = False) -> list:
    """Open ascii file for reading and get results of re.findall

    Parameters
    ----------
    regex : str
        Regular expression
    filepath : str
        Path of ascii text file to search
    skip_exist : bool
        True to skip searching through a file that doesnt exist

    Returns
    -------
    list
        Results from re.findall function
    """
    if (not path.exists(filepath)) and skip_exist:
        return []
    try:
        with open(filepath, "r", encoding="utf-8") as file:
            filetext = file.read()
    except UnicodeDecodeError as error:
        raise IOError from error
    results = findall(regex, filetext)
    return results


def get_clean_pyfiles(logpath: str) -> list[str]:
    """Get a list of already cleaned .py files from previous cleandoc run so
    cleaning functions can be skipped.

    Parameters
    ----------
    logpath : str
        Full path of previous log file output by cleandoc
    Returns
    -------
    list[str]
        List of .py files in nested directory which are still clean
    """
    clean_pyfiles = []
    regex = r"(\d\d-\d\d-\d\d \d\d:\d\d:\d\d).*File is Clean: (.*)"
    results = findall_infile(regex, logpath, skip_exist=True)
    for result in results:
        if not check_modified(result[1], result[0]):
            clean_pyfiles.append(result[1])
    return clean_pyfiles


def config_log(logpath: str, logname: str = "cleandoc"):
    """Configure log file using logging module

    Parameters
    ----------
    logpath : str
        Path of cleandoc log file to create
    logname : str
        Name for logger

    Returns
    -------
    logger object

    """
    if logging.getLogger(logname).hasHandlers():
        return logging.getLogger(logname)
    if path.exists(logpath):
        remove(logpath)
    logging.basicConfig(
        filename=logpath,
        filemode="a",
        format="%(asctime)s %(name)s %(levelname)s %(message)s",
        datefmt="%d-%m-%y %H:%M:%S",
        level=logging.INFO,
    )
    logger = logging.getLogger(logname)
    logger.addHandler(logging.StreamHandler(stdout))
    return logger


# if __name__ == "__main__":



# -*- coding: utf-8 -*-
"""
Created on Sun Jul  2 12:18:08 2023

@author: jkris
"""

from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as readme:
    readme_text = readme.read()

setup(
    name="cleandoc",
    version="0.0.1",
    description=(
        "Python package leveraging doq, black, pylint, mypy and sphinx"
        + " to automatically clean and document python code."
    ),
    long_description=readme_text,
    long_description_content_type="text/markdown",
    author="Jason Krist",
    author_email="jkrist2696@gmail.com",
    url="https://github.com/jkrist2696/cleandoc",
    license="GNU GPLv3",
    packages=find_packages(where="src"),
    install_requires=[
        "black>=23.3.0",
        "mypy>=1.4.1",
        "pylint>=2.17.4",
        "Sphinx>=6.2.1",
        "sphinx_rtd_theme>=1.2.2",
        "doq>=0.9.1",
    ],
    package_dir={"cleandoc": "src/cleandoc"},
    entry_points={"console_scripts": ["cleandoc=cleandoc.cli:main"]},
    python_requires=">=3.8",
)
